C=======================================================================
C=======================================================================
C
C    University of Maryland Baltimore County [UMBC]
C
C    NAST
C
C    CALPAR
C
!F77====================================================================


!ROUTINE NAME:
C    CALPAR


!ABSTRACT:
C    Calculate the fast transmittance code temperature/amount/angle
C    dependent variables for a profile.


!CALL PROTOCOL:
C    CALPAR ( LBOT, RTEMP,  RFAMNT, RWAMNT, ROAMNT, RCAMNT,
C  $                PTEMP,  PFAMNT, PWAMNT, POAMNT, PCAMNT,
C  $          PRES,   SECANG, FIXMUL, CONPRD,
C  $          FPRED1, FPRED2, FPRED3, FPRED4,
C  $          WPRED1, WPRED2, WPRED3, WPRED4,
C  $          OPRED1, OPRED2, OPRED3, OPRED4,
C  $          CPRED4 )


!INPUT PARAMETERS:
C    type      name    purpose                     units
C    --------  ------  --------------------------  ---------------------
C    INTEGER   LBOT    bottom layer number         none
C    REAL arr  PCAMNT  profile CO amount           k.mol/cm2
C    REAL arr  PTEMP   profile temperature         K
C    REAL arr  PFAMNT  profile fixed gases amount  k.mol/cm2
C    REAL arr  POAMNT  profile ozone amount        k.mol/cm2
C    REAL arr  PRES    layer pressures             atm
C    REAL arr  PWAMNT  profile water amount        k.mol/cm2
C    REAL arr  RCAMNT  reference CO amount         k.mol/cm2
C    REAL arr  RTEMP   reference temperature       K
C    REAL arr  RFAMNT  reference fixed amount      k.mol/cm2
C    REAL arr  ROAMNT  reference ozone amount      k.mol/cm2
C    REAL arr  RWAMNT  reference water amount      k.mol/cm2
C    REAL arr  SECANG  secant of path angle        none


!OUTPUT PARAMETERS:
C    type      name    purpose                     units
C    --------  ------  --------------------------  ---------------------
C    REAL arr  CPRED4  carbon monoxide pred set4   various
C    REAL arr  FIXMUL  fixed amount multiplier     none
C    REAL arr  FPRED1  fixed predictors set1       various
C    REAL arr  FPRED2  fixed predictors set2       various
C    REAL arr  FPRED3  fixed predictors set3       various
C    REAL arr  FPRED4  fixed predictors set4       various
C    REAL arr  OPRED1  ozone predictors set1       various
C    REAL arr  OPRED2  ozone predictors set2       various
C    REAL arr  OPRED3  ozone predictors set2       various
C    REAL arr  OPRED4  ozone predictors set4       various
C    REAL arr  WPRED1  water predictors set1       various
C    REAL arr  WPRED2  water predictors set2       various
C    REAL arr  WPRED3  water predictors set3       various
C    REAL arr  WPRED4  water predictors set4       various


!INPUT/OUTPUT PARAMETERS:
C    none


!RETURN VALUES:
C    none


!PARENT(S):
C    USEFAST


!ROUTINES CALLED:
C    none


!FILES ACCESSED:
C    incFTC.f : include file of parameter statements accessed during
C       compilation only.


!COMMON BLOCKS
C    none


!DESCRIPTION:
C    August 1998 version of the 65 layer NAST Fast Transmittance
C    Code by Howard Motteler/Larrabee Strow/Scott Hannon.
C
C    Fast transmittace algorithm predictors consisting of various gas
C    amount and temperature ratios and offsets relative to a reference
C    profile are calculated.
C
C    ===================================================================
C    The FTC profile variables computed for each layer are:
C
C    ---------------------------------
C    CONPRD: water continuum predictors (4 terms)
C       1) a*W/Tr^2   2) a*(W/Tr^2)^2   3) a*W/Tr  4) a*W^2/Tr
C
C    -------------------------------
C    Fixed predictors
C
C    FPRED1: FOW bf (11 terms):
C       1) a        2) Tr       3) a*Tr    4) a^2
C       5) a*Tr^2   6) Tr^2     7) a*Trz   8) a*Trz/Tr
C       9) a^2*Tr  10) sqrt(a) 11) Trz
C
C    FPRED2: FOW bo (8 terms):
C       1) a        2) Tr       3) a*Tr    4) a^2
C       5) a*Tr^2   6) Tr^2     7) a*Trz   8) a*Trz/Tr
C
C    FPRED3: FOW bw (8 terms):
C       1) a        2) Tr       3) a*Tr    4) a^2
C       5) a*Tr^2   6) Tr^2     7) a*Trz   8) sqrt(a)
C
C    FPRED4: FCOW sun (11 terms):
C       1) a        2) Tr       3) a*Tr    4) a^2
C       5) a*Tr^2   6) Tr^2     7) a*Trz   8) a*Trz/Tr
C       9) a^2*Tr  10) sqrt(a) 11) Trz
C
C    ---------------------------------
C    Water predictors
C
C    WPRED1: FOW bf (7 terms):
C       1) W*a           2) W*a)^3/2        3) W*a*dT
C       4) (W*a)^2       5) (W*a)^3/2*dT    6) (W*a)^5/4
C       7) W*a^2
C
C    WPRED2: FOW bo (11 terms):
C       1) W*a             2) (W*a)^3/2     3) W*a*dT
C       4) W*a*Ox*a        5) (W*a)^2       6) (W*a)^5/4
C       7) (W*a)^3/2*dT    8) W*a*W/Wz      9) (W*a)^3
C      10) W*a^2          11) (W*a)^3/2*W/Wz
C
C    WPRED3: FOW bo (13 terms):
C       1) W*a             2) sqrt(W*a)     3) W*a*dT
C       4) (W*a)^2         5) sqrt(W*a)*dT  6) (W*a)^3
C       7) W*a^2           8) W*a*W/Wz      9) sqrt(W*a)*W/Wz
C      10) root^4(W*a)    11) (W*a)^2*W/Wz 12) (W*a)^3/4
C      13) W^2*a
C
C    WPRED4: FCOW sun (13 terms):
C       1) W*a             2) W             3) sqrt(W*a)
C       4) W*a*dT          5) (W*a)^2       6) sqrt(W*a)*dT
C       7) root^4(W*a)     8) W*a*W/Wz      9) W*a^2
C      10) (W*a)^3        11) W*a*Cz*a     12) sqrt(W*a)*W/Wz
C      13) W*a^2*dT
C
C    ---------------------------
C    Ozone predictors
C
C    OPRED1: FOW bf (5 terms):
C       1) O*a             2) sqrt(O*a)     3) O*a*dT
C       4) (O*a)^2         5) O*a^2
C
C    OPRED2: FOW (10 terms):
C       1) O*a             2) sqrt(O*a)     3) O*a*dT
C       4) (O*a)^2         5) sqrt(O*a)*dT  6) O*a*O/Ox
C       7) sqrt(O*a)*O/Ox  8) (O*a)^2*O/Ox  9) O*a*sqrt(Ox*a)
C      10) O*a*TOz*a
C
C    OPRED3: FOW sun bw (3 terms):
C       1) O*a             2) sqrt(O*a)     3) O*a*dT
C
C    OPRED4: FCOW sun (9 terms):
C       1) O*a             2) sqrt(O*a)     3) O*a*dT
C       4) (O*a)^2         5) sqrt(O*a)*dT  6) O*a*O/Ox
C       7) (O*a)^3         8) O             9) (O*a)^3/2
C
C    ---------------------------
C    CO predictors
C
C    CPRED4: FCOW (11 terms):
C       1) C*a             2) sqrt(C*a)       3) C*a*dT
C       4) (C*a)^2         5) C*a*C/Cz        6) sqrt(C*a)*dT
C       7) root^4(C*a)     8) sqrt(C*a)*C/Cz  9) C
C      10) C*a^2          11) sqrt(C*a)*a
C
C    -----
C    where:
C    "a" is the secant of the viewing angle SECANG
C    "Tr" is the temperature ratio PTEMP/RTEMP
C    "Trz" is the pressure weighted temperature ratio above, i.e.
C      the sum i=2 to i=L of { P(i) * ( P(i) -  P(i-1) )* Tr(i-1) }
C      where "P" is the pressure PRES and "L" is the layer number, and
C      Trz(L=1)=0
C    "W" is the water amount ratio PWAMNT/RWAMNT
C    "dT" is the temperature offset PTEMP-RTEMP
C    "Wz" is the pressure weighted water amount above ratio, the
C      sum i=1 to i=L of { P(i) * ( (P(i)-P(i-1) ) * PWAMNT(i) },
C      divided by the same sum except using RWAMNT instead of PWAMNT.
C      For these sums, term P(0) is defined as P(0)=2*P(1) - P(2).
C    "O" is the ozone amount ratio POAMNT/ROAMNT
C    "Oz" is the pressure weighted ozone amount above ratio, the
C      sum i=1 to i=L of { P(i) * ( (P(i)-P(i-1) ) * POAMNT(i) },
C      divided by the same sum except using ROAMNT instead of POAMNT.
C      For these sums, term P(0) is defined as P(0)=2*P(1) - P(2)
C    "Ox" is the unweighted ozone amount above ratio, the
C      sum i=1 to i=L of { POAMNT(i) },
C      divided by the same sum except using ROAMNT instead of POAMNT.
C      For these sums, term P(0) is defined as P(0)=2*P(1) - P(2).
C    "TOz" is the pressure and ozone weighted temperature ratio above,
C      sum i=2 to i=L of { P(i) * ( P(i)-P(i-1) )* dT(i-1) * O(i-1) }
C      and TOz(L=1)=0
C    "C" is the CO amount ratio PCAMNT/RCAMNT
C    "Cz" is the pressure weighted CO amount above ratio, the
C      sum i=1 to i=L of { P(i) * ( (P(i)-P(i-1) ) * PCAMNT(i) },
C      divided by the same sum except using RWAMNT instead of PCAMNT.
C      For these sums, term P(0) is defined as P(0)=2*P(1) - P(2).
C
C    -----------------------------------------------------
C    FIXMUL: the not-quite-fixed "fixed" amount multiplier:
C       FIXMUL = PFAMNT/RFAMNT
C       This value should be close to (within a few percent of) 1.0.
C    ===================================================================


!ALGORITHM REFERENCES:
C    none


!KNOWN BUGS AND LIMITATIONS:
C    Assumes vaguely realistic profile amounts and temperatures, else
C    there might be divide by zero problems, etc.


!ROUTINE HISTORY:
C    Date         Programmer      Comments
C    -----------  --------------  --------------------------------------
C     7 Aug 1998  Scott Hannon    Created based on Jul98 HIS version
C    26 Aug 1998  Scott Hannon    Added LBOT to replace MAXLAY in loop


!END====================================================================

C      =================================================================
       SUBROUTINE CALPAR (
     $    LBOT, RTEMP,  RFAMNT, RWAMNT, ROAMNT, RCAMNT,
     $          PTEMP,  PFAMNT, PWAMNT, POAMNT, PCAMNT,
     $    PRES,   SECANG, FIXMUL, CONPRD,
     $    FPRED1, FPRED2, FPRED3, FPRED4,
     $    WPRED1, WPRED2, WPRED3, WPRED4,
     $    OPRED1, OPRED2, OPRED3, OPRED4, CPRED4 )
C      =================================================================


C-----------------------------------------------------------------------
C      IMPLICIT NONE
C-----------------------------------------------------------------------
       IMPLICIT NONE


C-----------------------------------------------------------------------
C      INCLUDE FILES
C-----------------------------------------------------------------------
#include "incFTC.f"


C-----------------------------------------------------------------------
C      EXTERNAL FUNCTIONS
C-----------------------------------------------------------------------
C      none


C-----------------------------------------------------------------------
C      ARGUMENTS
C-----------------------------------------------------------------------
C      Input
       INTEGER   LBOT
       REAL  RTEMP(MAXLAY)
       REAL RFAMNT(MAXLAY)
       REAL RWAMNT(MAXLAY)
       REAL ROAMNT(MAXLAY)
       REAL RCAMNT(MAXLAY)
       REAL  PTEMP(MAXLAY)
       REAL PFAMNT(MAXLAY)
       REAL PWAMNT(MAXLAY)
       REAL POAMNT(MAXLAY)
       REAL PCAMNT(MAXLAY)
       REAL   PRES(MAXLAY)
       REAL SECANG(MAXLAY)
C
C      Output
       REAL FIXMUL(MAXLAY)
       REAL CONPRD( N1CON,MAXLAY)
       REAL FPRED1( N1FIX,MAXLAY)
       REAL FPRED2( N2FIX,MAXLAY)
       REAL FPRED3( N3FIX,MAXLAY)
       REAL FPRED4( N4FIX,MAXLAY)
       REAL WPRED1( N1H2O,MAXLAY)
       REAL WPRED2( N2H2O,MAXLAY)
       REAL WPRED3( N3H2O,MAXLAY)
       REAL WPRED4( N4H2O,MAXLAY)
       REAL OPRED1(  N1O3,MAXLAY)
       REAL OPRED2(  N2O3,MAXLAY)
       REAL OPRED3(  N3O3,MAXLAY)
       REAL OPRED4(  N4O3,MAXLAY)
       REAL CPRED4(  N4CO,MAXLAY)


C-----------------------------------------------------------------------
C      LOCAL VARIABLES
C-----------------------------------------------------------------------
       INTEGER      I
       INTEGER      L
       REAL    PDP
       REAL  PNORM
       REAL     DT
       REAL     TR
       REAL     TZ
       REAL    TRZ
       REAL    A_F
       REAL    A_W
       REAL  WZREF
       REAL     WZ
       REAL   AZ_W
       REAL    A_O
       REAL  XZREF
       REAL     XZ
       REAL   XZ_O
       REAL  OZREF
       REAL     OZ
       REAL   AZ_O
       REAL    TOZ
       REAL  TAZ_O
       REAL     CZ
       REAL  CZREF
       REAL    A_C
       REAL   AZ_C
       REAL TJUNKS
       REAL WJUNKA
       REAL WJUNKR
       REAL WJUNKS
       REAL WJUNKZ
       REAL WJUNK4
       REAL OJUNKA
       REAL OJUNKR
       REAL OJUNKZ
       REAL OJUNKX
       REAL CJUNKA
       REAL CJUNKR
       REAL CJUNKS
       REAL CJUNKZ


C-----------------------------------------------------------------------
C      SAVE STATEMENTS
C-----------------------------------------------------------------------
C      none


C***********************************************************************
C***********************************************************************
C                    EXECUTABLE CODE
C***********************************************************************
C***********************************************************************
C
C
C      Initialize the sum terms to zero
       PNORM=0.0E+0
       TZ=0.0E+0
       WZREF=0.0E+0
       WZ=0.0E+0
       XZREF=0.0E+0
       XZ=0.0E+0
       OZREF=0.0E+0
       OZ=0.0E+0
       TOZ=0.0E+0
       CZREF=0.0E+0
       CZ=0.0E+0
C
C      --------------------
C      Loop over the layers
C      --------------------
       DO L=1,LBOT
C
C         ---------------------------
C         Calculate the basic profile
C         dependent predictors.
C         ---------------------------
C
          IF (L .EQ. 1) THEN
             PDP=PRES(1)*( PRES(2) - PRES(1))
             TRZ=0.0E+0
             TAZ_O=0.0E+0
          ELSE
             PDP=PRES(L)*( PRES(L) - PRES(L-1) )
             PNORM=PNORM + PDP
C
C            Note: TRZ, TOZ, and TMZ use layer-above terms
             TZ=TZ + PDP*TR
             TRZ=TZ/PNORM
C
             TOZ=TOZ + PDP*DT*A_O
             TAZ_O=TOZ/PNORM
C
          ENDIF
C
C         Temperature terms
          DT=PTEMP(L) - RTEMP(L)
          TR=PTEMP(L)/RTEMP(L)
C
C         Fixed term
          A_F=PFAMNT(L)/RFAMNT(L)
C
C         Water terms
          A_W=PWAMNT(L)/RWAMNT(L)
          WZREF=WZREF + PDP*RWAMNT(L)
          WZ=WZ + PDP*PWAMNT(L)
          AZ_W=WZ/WZREF
C
C         Ozone terms
          A_O=POAMNT(L)/ROAMNT(L)
          XZREF=XZREF + ROAMNT(L)
          XZ=XZ + POAMNT(L)
          XZ_O=XZ/XZREF
          OZREF=OZREF + PDP*ROAMNT(L)
          OZ=OZ + PDP*POAMNT(L)
          AZ_O=OZ/OZREF
C
C         CO terms
          A_C=PCAMNT(L)/RCAMNT(L)
          CZREF=CZREF + PDP*RCAMNT(L)
          CZ=CZ + PDP*PCAMNT(L)
          AZ_C=CZ/CZREF
C
C         ----------------------
C         Load up the predictors
C         ----------------------
C
C         -----
C         Fixed
C         -----
          TJUNKS=TR*TR
          FIXMUL(L)=A_F
C
C         Fixed predictors for set 1 = FOW bf
          FPRED1( 1,L)=SECANG(L)
          FPRED1( 2,L)=TR
          FPRED1( 3,L)=SECANG(L)*TR
          FPRED1( 4,L)=SECANG(L)*SECANG(L)
          FPRED1( 5,L)=SECANG(L)*TJUNKS
          FPRED1( 6,L)=TJUNKS
          FPRED1( 7,L)=SECANG(L)*TRZ
          FPRED1( 8,L)=SECANG(L)*TRZ/TR
          FPRED1( 9,L)=SECANG(L)*SECANG(L)*TR
          FPRED1(10,L)=SQRT(SECANG(L))
          FPRED1(11,L)=TRZ
C
C         Fixed predictors for set 2 = FOW bo
          FPRED2( 1,L)=SECANG(L)
          FPRED2( 2,L)=TR
          FPRED2( 3,L)=SECANG(L)*TR
          FPRED2( 4,L)=SECANG(L)*SECANG(L)
          FPRED2( 5,L)=SECANG(L)*TJUNKS
          FPRED2( 6,L)=TJUNKS
          FPRED2( 7,L)=SECANG(L)*TRZ
          FPRED2( 8,L)=SECANG(L)*TRZ/TR
C
C         Fixed predictors for set 3 = FOW bw
          FPRED3( 1,L)=SECANG(L)
          FPRED3( 2,L)=TR
          FPRED3( 3,L)=SECANG(L)*TR
          FPRED3( 4,L)=SECANG(L)*SECANG(L)
          FPRED3( 5,L)=SECANG(L)*TJUNKS
          FPRED3( 6,L)=TJUNKS
          FPRED3( 7,L)=SECANG(L)*TRZ
          FPRED3( 8,L)=SQRT( SECANG(L) )
C
C         Fixed predictors for set 4 = FCOW sun
          FPRED4( 1,L)=SECANG(L)
          FPRED4( 2,L)=TR
          FPRED4( 3,L)=SECANG(L)*TR
          FPRED4( 4,L)=SECANG(L)*SECANG(L)
          FPRED4( 5,L)=SECANG(L)*TJUNKS
          FPRED4( 6,L)=TJUNKS
          FPRED4( 7,L)=SECANG(L)*TRZ
          FPRED4( 8,L)=SECANG(L)*TRZ/TR
          FPRED4( 9,L)=SECANG(L)*SECANG(L)*TR
          FPRED4(10,L)=SQRT(SECANG(L))
          FPRED4(11,L)=TRZ
C
C
C         -----
C         Ozone
C         -----
          OJUNKA=SECANG(L)*A_O
          OJUNKR=SQRT( OJUNKA )
          OJUNKZ=OJUNKA/XZ_O
          OJUNKX=SECANG(L)*XZ_O
C
C         Ozone predictors for set 1 = FOW bf
          OPRED1( 1,L)=OJUNKA
          OPRED1( 2,L)=OJUNKR
          OPRED1( 3,L)=OJUNKA*DT
          OPRED1( 4,L)=OJUNKA*OJUNKA
          OPRED1( 5,L)=OJUNKA*SECANG(L)
C
C         ozone predictors for set 2 = FOW bo
          OPRED2( 1,L)=OJUNKA
          OPRED2( 2,L)=OJUNKR
          OPRED2( 3,L)=OJUNKA*DT
          OPRED2( 4,L)=OJUNKA*OJUNKA
          OPRED2( 5,L)=OJUNKR*DT
          OPRED2( 6,L)=OJUNKZ*A_O
          OPRED2( 7,L)=OJUNKR*A_O/XZ_O
          OPRED2( 8,L)=OJUNKA*OJUNKZ*A_O
          OPRED2( 9,L)=OJUNKA*SQRT( OJUNKX )
          OPRED2(10,L)=OJUNKA*TAZ_O*SECANG(L)
C
C         ozone predictors for set 3 = FOW bw
          OPRED3( 1,L)=OJUNKA
          OPRED3( 2,L)=OJUNKR
          OPRED3( 3,L)=OJUNKA*DT
C
C         ozone predictors for set 4 = FCOW
          OPRED4( 1,L)=OJUNKA
          OPRED4( 2,L)=OJUNKR
          OPRED4( 3,L)=OJUNKA*DT
          OPRED4( 4,L)=OJUNKA*OJUNKA
          OPRED4( 5,L)=OJUNKR*DT
          OPRED4( 6,L)=OJUNKA*A_O/XZ_O
          OPRED4( 7,L)=OJUNKA**3
          OPRED4( 8,L)=A_O
          OPRED4( 9,L)=OJUNKA*OJUNKR
C
C
C         -----
C         Water
C         -----
          WJUNKA=SECANG(L)*A_W
          WJUNKR=SQRT( WJUNKA )
          WJUNKS=WJUNKA*WJUNKA
          WJUNKZ=WJUNKA*A_W/AZ_W
          WJUNK4=SQRT( WJUNKR )
C
C         Water predictors for set 1 = FOW bf
          WPRED1( 1,L)=WJUNKA
          WPRED1( 2,L)=WJUNKA*WJUNKR
          WPRED1( 3,L)=WJUNKA*DT
          WPRED1( 4,L)=WJUNKS
          WPRED1( 5,L)=WJUNKA*WJUNKR*DT
          WPRED1( 6,L)=WJUNKA*WJUNK4
          WPRED1( 7,L)=WJUNKA*SECANG(L)
C
C         water predictors for set 2 = FOW bo
          WPRED2( 1,L)=WJUNKA
          WPRED2( 2,L)=WJUNKA*WJUNKR
          WPRED2( 3,L)=WJUNKA*DT
          WPRED2( 4,L)=WJUNKA*OJUNKX
          WPRED2( 5,L)=WJUNKS
          WPRED2( 6,L)=WJUNKA*WJUNK4
          WPRED2( 7,L)=WJUNKA*WJUNKR*DT
          WPRED2( 8,L)=WJUNKZ
          WPRED2( 9,L)=WJUNKA*WJUNKS
          WPRED2(10,L)=WJUNKA*SECANG(L)
          WPRED2(11,L)=WJUNKZ*WJUNKR
C
C         Water predictors for set 3 = FOW bw
          WPRED3( 1,L)=WJUNKA
          WPRED3( 2,L)=WJUNKR
          WPRED3( 3,L)=WJUNKA*DT
          WPRED3( 4,L)=WJUNKS
          WPRED3( 5,L)=WJUNKR*DT
          WPRED3( 6,L)=WJUNKS*WJUNKA
          WPRED3( 7,L)=WJUNKA*SECANG(L)
          WPRED3( 8,L)=WJUNKZ
          WPRED3( 9,L)=WJUNKZ/WJUNKR
          WPRED3(10,L)=WJUNK4
          WPRED3(11,L)=WJUNKZ*WJUNKA
          WPRED3(12,L)=WJUNKA/WJUNK4
          WPRED3(13,L)=WJUNKA*A_W
C
C         Water predictors for set 4 = FCOW
          WPRED4( 1,L)=WJUNKA
          WPRED4( 2,L)=A_W
          WPRED4( 3,L)=WJUNKR
          WPRED4( 4,L)=WJUNKA*DT
          WPRED4( 5,L)=WJUNKA*WJUNKA
          WPRED4( 6,L)=WJUNKR*DT
          WPRED4( 7,L)=WJUNK4
          WPRED4( 8,L)=WJUNKZ
          WPRED4( 9,L)=WJUNKA*SECANG(L)
          WPRED4(10,L)=WJUNKS*WJUNKA
          WPRED4(11,L)=WJUNKA*AZ_C*SECANG(L)
          WPRED4(12,L)=WJUNKZ/WJUNKR
          WPRED4(13,L)=WJUNKA*DT*SECANG(L)
C
C
C         ---------------
C         Water continuum
C         ---------------
          CONPRD(1,L)=WJUNKA/TJUNKS
          CONPRD(2,L)=CONPRD(1,L)*A_W/TJUNKS
          CONPRD(3,L)=WJUNKA/TR
          CONPRD(4,L)=CONPRD(3,L)*A_W
C
C
C         --
C         CO
C         --
          CJUNKA=SECANG(L)*A_C
          CJUNKR=SQRT( CJUNKA )
          CJUNKS=CJUNKA*CJUNKA
          CJUNKZ=CJUNKA*A_C/AZ_C
C
C         CO predictors for set 4 = FCOW
          CPRED4( 1,L)=CJUNKA
          CPRED4( 2,L)=CJUNKR
          CPRED4( 3,L)=CJUNKA*DT
          CPRED4( 4,L)=CJUNKS
          CPRED4( 5,L)=CJUNKZ
          CPRED4( 6,L)=CJUNKR*DT
          CPRED4( 7,L)=SQRT( CJUNKR )
          CPRED4( 8,L)=CJUNKZ/CJUNKR
          CPRED4( 9,L)=A_C
          CPRED4(10,L)=CJUNKA*SECANG(L)
          CPRED4(11,L)=CJUNKR*SECANG(L)

       ENDDO
C      End loop over layers
C
       RETURN
       END
