C=======================================================================
C=======================================================================
C
C    University of Maryland Baltimore County [UMBC]
C
C    NAST
C
C    CALT1 (for set1 = FOW bf)
C
!F77====================================================================


!ROUTINE NAME:
C    CALT1


!ABSTRACT:
C    Calculate the fast transmittance tau for set1 using the predictors
C    and the fast transmittance coefficients.


!CALL PROTOCOL:
C    CALT1 (LTAU, INDCHN, NLAY, BLMULT, NCHN1, CLIST1, LCHNON, COEF1,
C       FIXMUL, CONPD1, FPRED1, WPRED1, OPRED1, TAU, TAUZ )


!INPUT PARAMETERS:
C    type      name    purpose                     units
C    --------  ------  --------------------------  ---------------------
C    LOGICAL   LTAU    calc all layer trans?       none
C    INT arr   INDCHN  channel indices             none
C    INTEGER   NLAY    Number of layers to bottom  none
C    REAL      BLMULT  bottom layer multiplier     none
C    INTEGER   NCHN1   set1 number of channels     none
C    INT arr   CLIST1  set channel list            none
C    LOG arr   LCHNON  chan calc on/off switch     none
C    REAL arr  COEF1   set1 fast trans coefs       various
C    REAL arr  FIXMUL  fixed amount mult (~1.0)    none
C    REAL arr  CONPD1  set1 H2O continuum preds    various
C    REAL arr  FPRED1  set1 fixed gases preds      various
C    REAL arr  WPRED1  set1 water predictors       various
C    REAL arr  OPRED1  set1 ozone predictors       various


!OUTPUT PARAMETERS:
C    type      name    purpose                     units
C    --------  ------  --------------------------  ---------------------
C    REAL arr  TAU     effective layer trans       none
C    REAL arr  TAUZ    surface-to-space trans      none


!INPUT/OUTPUT PARAMETERS:
C    none


!RETURN VALUES:
C    none


!PARENT(S):
C    USEFAST


!ROUTINES CALLED:
C    none


!FILES ACCESSED:
C    incFTC.f : include file of parameter statements accessed during
C       compilation only.


!COMMON BLOCKS
C    none


!DESCRIPTION:
C    August 1998 version of the NAST layer AIRS Fast Transmittance
C    Code by Howard Motteler/Larrabee Strow/Scott Hannon.
C
C    The FTC coefficents and profile FTC variables are multiplied
C    together and summed to calculate the effecttive layer
C    transmittances. Fixed, water, and ozone transmittances are each
C    checked individually to be sure they give 0 < tau < 1.
C
C    ===================================================================
C    Loops downward over all the layers for each of the NCHN1 channels
C    to compute the layer transmittances TAU.
C
C    The water continuum absorption coefficient is:
C       k_con = the sum i=1 to 4 of { COEF(i)*CONPRD(i) }
C
C    The layer effective fixed gas absorption coefficient is:
C       k_fixed = the sum i=1 to 11 of { COEF(4+i)*FPRED(i) }
C
C    The layer effective ozone absorption coefficient is:
C       k_ozone = the sum i=1 to 5 of { COEF(4+11+i)*OPRED(i) }
C
C    The layer effective water lines absorption coefficient is:
C       k_water = the sum i=1 to 7 of { COEF(4+11+5+i)*WPRED(i) }
C
C    where
C      "COEF" are the fast transmittance coefficients COEF1
C      "CONPRD" are the water continuum predictors CONPRD
C      "FPRED" are the fixed gases predictors FPRED
C      "OPRED" are the ozone predictors OPRED1
C      "WPRED" are the water lines predictors WPRED1
C
C    The total layer effective transmittance is:
C       TAU = exp( -[ k_con + k_fixed + k_water + k_ozone ])
C
C    To help speed up the exponential calculations, we use our own
C    "EXP" replacement function called QIKEXP which uses just the
C    first two or three series expansion terms for exp(x) if x is
C    suitably small.
C    ===================================================================


!ALGORITHM REFERENCES:
C    none


!KNOWN BUGS AND LIMITATIONS:
C    none


!ROUTINE HISTORY:
C    Date         Programmer      Comments
C    -----------  --------------  --------------------------------------
C    14 Aug 1998  Scott Hannon    Created based on Jul98 HIS version
C    10 Sep 1998  Scott Hannon    Added LCHNON


!END====================================================================

C      =================================================================
       SUBROUTINE CALT1 ( LTAU, INDCHN, NLAY, BLMULT, NCHN1, CLIST1,
     $    LCHNON, COEF1, FIXMUL, CONPD1, FPRED1, WPRED1, OPRED1,
     $    TAU, TAUZ)
C      =================================================================
C
C-----------------------------------------------------------------------
C      IMPLICIT NONE
C-----------------------------------------------------------------------
       IMPLICIT NONE


C-----------------------------------------------------------------------
C      INCLUDE FILES
C-----------------------------------------------------------------------
#include "incFTC.f"


C-----------------------------------------------------------------------
C      EXTERNAL FUNCTIONS
C-----------------------------------------------------------------------
C      QIKEXP


C-----------------------------------------------------------------------
C      ARGUMENTS
C-----------------------------------------------------------------------
C      Input
       LOGICAL   LTAU
       INTEGER INDCHN(MXCHAN)
       INTEGER   NLAY
       INTEGER  NCHN1
       INTEGER CLIST1(MXCHN1)
       LOGICAL LCHNON(MXCHAN)
       REAL BLMULT
       REAL  COEF1(N1COEF,MAXLAY,MXCHN1)
       REAL FIXMUL(MAXLAY)
       REAL CONPD1( N1CON,MAXLAY)
       REAL FPRED1( N1FIX,MAXLAY)
       REAL WPRED1( N1H2O,MAXLAY)
       REAL OPRED1(  N1O3,MAXLAY)
C
C      Output
       REAL    TAU(MAXLAY,MXCHAN)
       REAL   TAUZ(MXCHAN)


C-----------------------------------------------------------------------
C      LOCAL VARIABLES
C-----------------------------------------------------------------------
       INTEGER      I
       INTEGER   ILAY
       INTEGER      J
       REAL   KCON
       REAL   KFIX
       REAL   KOZO
       REAL   KWAT
       REAL     KZ
       REAL KLAYER
C
C      for function QIKEXP
       REAL QIKEXP


C-----------------------------------------------------------------------
C      SAVE STATEMENTS
C-----------------------------------------------------------------------
C      none


C***********************************************************************
C***********************************************************************
C                    EXECUTABLE CODE
C***********************************************************************
C***********************************************************************
C
C      ---------------------------
C      Loop on channel (frequency)
C      ---------------------------
       DO I=1,NCHN1
C
C       Only do calc if channel is turned "on"
        IF (LCHNON(CLIST1(I))) THEN
C
C         Index for TAU
          J=INDCHN( CLIST1(I) )
C
C         Initialize the layer-to-space optical depth
          KZ=0.0E+0
C
C         ------------------------------
C         Loop on layers (top to bottom)
C         ------------------------------
          DO ILAY=1,NLAY
C
C
C            ---------------------------
C            Compute the water continuum
C            ---------------------------
             KCON=( COEF1(1,ILAY,I)*CONPD1(1,ILAY) ) +
     $            ( COEF1(2,ILAY,I)*CONPD1(2,ILAY) ) +
     $            ( COEF1(3,ILAY,I)*CONPD1(3,ILAY) ) +
     $            ( COEF1(4,ILAY,I)*CONPD1(4,ILAY) )
C
             IF (KCON .LT. 0.0E+0) THEN
                KCON=0.0E+0
             ELSEIF (KCON .GT. 1.0E+1) THEN
                KCON=1.0E+1
             ENDIF
C
C
C            -----------------------------
C            Calc the fixed gases abs coef
C            -----------------------------
             KFIX=( COEF1( 5,ILAY,I)*FPRED1( 1,ILAY) ) +
     $            ( COEF1( 6,ILAY,I)*FPRED1( 2,ILAY) ) +
     $            ( COEF1( 7,ILAY,I)*FPRED1( 3,ILAY) ) +
     $            ( COEF1( 8,ILAY,I)*FPRED1( 4,ILAY) ) +
     $            ( COEF1( 9,ILAY,I)*FPRED1( 5,ILAY) ) +
     $            ( COEF1(10,ILAY,I)*FPRED1( 6,ILAY) ) +
     $            ( COEF1(11,ILAY,I)*FPRED1( 7,ILAY) ) +
     $            ( COEF1(12,ILAY,I)*FPRED1( 8,ILAY) ) +
     $            ( COEF1(13,ILAY,I)*FPRED1( 9,ILAY) ) +
     $            ( COEF1(14,ILAY,I)*FPRED1(10,ILAY) ) +
     $            ( COEF1(15,ILAY,I)*FPRED1(11,ILAY) )
C
cccc
c removed for comparing with unfit data
c             KFIX=KFIX*FIXMUL(ILAY)
cccc
C
             IF (KFIX .LT. 0.0E+0) THEN
                KFIX=0.0E+0
             ELSEIF (KFIX .GT. 1.0E+1) THEN
                KFIX=1.0E+1
             ENDIF
C
C
C            --------------------------
C            Compute the ozone abs coef
C            --------------------------
             KOZO=( COEF1(16,ILAY,I)*OPRED1(1,ILAY) ) +
     $            ( COEF1(17,ILAY,I)*OPRED1(2,ILAY) ) +
     $            ( COEF1(18,ILAY,I)*OPRED1(3,ILAY) ) +
     $            ( COEF1(19,ILAY,I)*OPRED1(4,ILAY) ) +
     $            ( COEF1(20,ILAY,I)*OPRED1(5,ILAY) )
C
             IF (KOZO .LT. 0.0E+0) THEN
                KOZO=0.0E+0
             ELSEIF (KOZO .GT. 1.0E+1) THEN
                KOZO=1.0E+1
             ENDIF
C
C
C            --------------------------
C            Compute the water abs coef
C            --------------------------
             KWAT=( COEF1(21,ILAY,I)*WPRED1( 1,ILAY) ) +
     $            ( COEF1(22,ILAY,I)*WPRED1( 2,ILAY) ) +
     $            ( COEF1(23,ILAY,I)*WPRED1( 3,ILAY) ) +
     $            ( COEF1(24,ILAY,I)*WPRED1( 4,ILAY) ) +
     $            ( COEF1(25,ILAY,I)*WPRED1( 5,ILAY) ) +
     $            ( COEF1(26,ILAY,I)*WPRED1( 6,ILAY) ) +
     $            ( COEF1(27,ILAY,I)*WPRED1( 7,ILAY) )
C
             IF (KWAT .LT. 0.0E+0) THEN
                KWAT=0.0E+0
             ELSEIF( KWAT .GT. 1.0E+1) THEN
                KWAT=1.0E+1
             ENDIF
C
C
C            ----------------------------------
C            Calc the total layer transmittance
C            ----------------------------------
c
ccccc
c This block is usually commented out and is only uncommented for
c testing purposes.
cc
c           kcon=0.0E+0
c           kfix=0.0E+0
c           kozo=0.0E+0
c           kwat=0.0E+0
ccccc
C
C            Calc the total layer effective optical depth
             KLAYER=KCON + KFIX + KOZO + KWAT
C
C            Adjust the optical depth of the bottom layer
             IF (ILAY .EQ. NLAY) KLAYER=BLMULT*KLAYER
C
             KZ=KZ + KLAYER
             IF (LTAU) TAU(ILAY,J)=QIKEXP(-KLAYER)
C
          ENDDO
C         End loop on levels
C
C         Convert KZ to TAUZ
          TAUZ(J)=QIKEXP(-KZ)
C
        ENDIF
C       End of if-then for LCHNON "on"
C
       ENDDO
C      End loops on channel number (frequency)
C
       RETURN
       END
