C=======================================================================
C=======================================================================
C
C    University of Maryland Baltimore County [UMBC]
C
C    NAST
C
C    CALT2 (for set2 = FOW bo)
C
!F77====================================================================


!ROUTINE NAME:
C    CALT2


!ABSTRACT:
C    Calculate the transmittance using the predictors and the fast
C    transmittance coefficients.


!CALL PROTOCOL:
C    CALT2 ( INDCHN, NLAY, BLMULT, NCHN2, CLIST2, LCHNON, COEF2, FIXMUL,
C       CONPD2, FPRED2, WPRED2, OPRED2, TAU, TAUZ )


!INPUT PARAMETERS:
C    type      name    purpose                     units
C    --------  ------  --------------------------  ---------------------
C    INT arr   INDCHN  channel indices             none
C    INTEGER   NLAY    number of layers to bottom  none
C    REAL      BLMULT  bottom layer multiplier     none
C    INTEGER   NCHN2   set2 number of channels     none
C    INT arr   CLIST2  set2 channel list           none
C    LOG arr   LCHNON  chan calc on/off switch     none
C    REAL arr  COEF2   set2 fast trans coefs       various
C    REAL arr  FIXMUL  fixed amount mult (~1.0)    none
C    REAL arr  CONPD2  set2 H2O continuum preds    various
C    REAL arr  FPRED2  set2 fixed gases preds      various
C    REAL arr  WPRED2  set2 water predictors       various
C    REAL arr  OPRED2  set2 ozone predictors       various


!OUTPUT PARAMETERS:
C    type      name    purpose                     units
C    --------  ------  --------------------------  ---------------------
C    REAL arr  TAU     effective layer trans       none
C    REAL arr  TAUZ    surface-to-space trans      none


!INPUT/OUTPUT PARAMETERS:
C    none


!RETURN VALUES:
C    none


!PARENT(S):
C    USEFAST


!ROUTINES CALLED:
C    none


!FILES ACCESSED:
C    incFTC.f : include file of parameter statements accessed during
C       compilation only.


!COMMON BLOCKS
C    none


!DESCRIPTION:
C    August 1998 version of the 65 layer NAST Fast Transmittance
C    Code by Howard Motteler/Larrabee Strow/Scott Hannon.
C
C    The fast trans coefficents and predictors are multiplied
C    together and summed to calculate the effecttive layer
C    transmittances. Fixed, water, and ozone transmittances are each
C    checked individually to be sure they give 0 < tau < 1.
C
C    ===================================================================
C    Loops downward over all the layers for each of the NCHN2 channels
C    to compute the layer transmittances TAU.
C
C    The water continuum absorption coefficient is:
C       k_con = the sum i=1 to 4 of { COEF(i)*CONPRD(i) }
C
C    The layer effective fixed gas absorption coefficient is:
C       k_fixed = the sum i=1 to 5 of { COEF(4+i)*FPRED(i) }
C
C    The layer effective ozone absorption coefficient is:
C       k_ozone = the sum i=1 to 10 of { COEF(4+5+i)*OPRED(i) }
C
C    The layer effective water lines absorption coefficient is:
C       k_water = the sum i=1 to 10 of { COEF(4+5+10+i)*WPRED(i) }
C
C    where
C      "COEF" are the fast transmittance coefficients COEF2
C      "CONPRD" are the water continuum predictors CONPRD
C      "FPRED" are the fixed gases predictors FPRED
C      "OPRED" are the ozone predictors OPRED2
C      "WPRED" are the water lines predictors WPRED2
C
C    The total layer effective transmittance is:
C       TAU = exp( -[ k_con + k_fixed + k_ozone + k_water])
C
C    To help speed up the exponential calculations, we use our own
C    "EXP" replacement function called QIKEXP which uses just the
C    first two or three series expansion terms for exp(x) if x is
C    suitably small.
C    ===================================================================


!ALGORITHM REFERENCES:
C    none


!KNOWN BUGS AND LIMITATIONS:
C    none


!ROUTINE HISTORY:
C    Date         Programmer      Comments
C    -----------  --------------  --------------------------------------
C     7 Aug 1998  Scott Hannon    Created based on Jul98 HIS version
C    10 Sep 1998  Scott Hannon    Added LCHNON


!END====================================================================

C      =================================================================
       SUBROUTINE CALT2 ( INDCHN, NLAY, BLMULT, NCHN2, CLIST2, LCHNON,
     $    COEF2, FIXMUL, CONPD2, FPRED2, WPRED2, OPRED2, TAU, TAUZ )

C      =================================================================
C
C-----------------------------------------------------------------------
C      IMPLICIT NONE
C-----------------------------------------------------------------------
       IMPLICIT NONE


C-----------------------------------------------------------------------
C      INCLUDE FILES
C-----------------------------------------------------------------------
#include "incFTC.f"


C-----------------------------------------------------------------------
C      EXTERNAL FUNCTIONS
C-----------------------------------------------------------------------
C      QIKEXP


C-----------------------------------------------------------------------
C      ARGUMENTS
C-----------------------------------------------------------------------
C      Input
       INTEGER INDCHN(MXCHAN)
       INTEGER   NLAY
       INTEGER  NCHN2
       INTEGER CLIST2(MXCHN2)
       LOGICAL LCHNON(MXCHAN)
       REAL BLMULT
       REAL  COEF2(N2COEF,MAXLAY,MXCHN2)
       REAL FIXMUL(MAXLAY)
       REAL CONPD2( N2CON,MAXLAY)
       REAL FPRED2( N2FIX,MAXLAY)
       REAL OPRED2(  N2O3,MAXLAY)
       REAL WPRED2( N2H2O,MAXLAY) 
C
C      Output
       REAL    TAU(MAXLAY,MXCHAN)
       REAL   TAUZ(MXCHAN)

C-----------------------------------------------------------------------
C      LOCAL VARIABLES
C-----------------------------------------------------------------------
       INTEGER      I
       INTEGER   ILAY
       INTEGER      J
       REAL   KCON
       REAL   KFIX
       REAL   KOZO
       REAL   KWAT
       REAL     KZ
       REAL KLAYER
C
C      for function QIKEXP
       REAL QIKEXP


C-----------------------------------------------------------------------
C      SAVE STATEMENTS
C-----------------------------------------------------------------------
C      none


C***********************************************************************
C***********************************************************************
C                    EXECUTABLE CODE
C***********************************************************************
C***********************************************************************
C
C      ---------------------------
C      Loop on channel (frequency)
C      ---------------------------
       DO I=1,NCHN2
C
C       Only do calc if channel is turned "on"
        IF (LCHNON(CLIST2(I))) THEN
C
C         Index for TAU
          J=INDCHN( CLIST2(I) )
C
C         Initialize the layer-to-space optical depth
          KZ=0.0E+0
C
C         ------------------------------
C         Loop on layers (top to ground)
C         ------------------------------
          DO ILAY=1,NLAY
C
C
C            ---------------------------
C            Compute the water continuum
C            ---------------------------
             KCON=( COEF2(1,ILAY,I)*CONPD2(1,ILAY) ) +
     $            ( COEF2(2,ILAY,I)*CONPD2(2,ILAY) ) +
     $            ( COEF2(3,ILAY,I)*CONPD2(3,ILAY) ) +
     $            ( COEF2(4,ILAY,I)*CONPD2(4,ILAY) )
C
             IF (KCON .LT. 0.0+0) THEN
                KCON=0.0E+0
             ELSEIF (KCON .GT. 1.0E+1) THEN
                KCON=1.0E+1
             ENDIF
C
C
C            -----------------------------
C            Calc the fixed gases abs coef
C            -----------------------------
             KFIX=( COEF2( 5,ILAY,I)*FPRED2(1,ILAY) ) +
     $            ( COEF2( 6,ILAY,I)*FPRED2(2,ILAY) ) +
     $            ( COEF2( 7,ILAY,I)*FPRED2(3,ILAY) ) +
     $            ( COEF2( 8,ILAY,I)*FPRED2(4,ILAY) ) +
     $            ( COEF2( 9,ILAY,I)*FPRED2(5,ILAY) ) +
     $            ( COEF2(10,ILAY,I)*FPRED2(6,ILAY) ) +
     $            ( COEF2(11,ILAY,I)*FPRED2(7,ILAY) ) +
     $            ( COEF2(12,ILAY,I)*FPRED2(8,ILAY) )
C
cccc
c removed for comparing with unfit data
c             KFIX=KFIX*FIXMUL(L)
cccc
C
             IF (KFIX .LT. 0.0E+0) THEN
                KFIX=0.0E+0
             ELSEIF (KFIX .GT. 1.0E+1) THEN
                KFIX=1.0E+1
             ENDIF
C
C
C            --------------------------
C            Compute the ozone abs coef
C            --------------------------
             KOZO=( COEF2(13,ILAY,I)*OPRED2( 1,ILAY) ) +
     $            ( COEF2(14,ILAY,I)*OPRED2( 2,ILAY) ) +
     $            ( COEF2(15,ILAY,I)*OPRED2( 3,ILAY) ) +
     $            ( COEF2(16,ILAY,I)*OPRED2( 4,ILAY) ) +
     $            ( COEF2(17,ILAY,I)*OPRED2( 5,ILAY) ) +
     $            ( COEF2(18,ILAY,I)*OPRED2( 6,ILAY) ) +
     $            ( COEF2(19,ILAY,I)*OPRED2( 7,ILAY) ) +
     $            ( COEF2(20,ILAY,I)*OPRED2( 8,ILAY) ) +
     $            ( COEF2(21,ILAY,I)*OPRED2( 9,ILAY) ) +
     $            ( COEF2(22,ILAY,I)*OPRED2(10,ILAY) )
C
             IF (KOZO .LT. 0.0E+0) THEN
                KOZO=0.0E+0
             ELSEIF (KOZO .GT. 1.0E+1) THEN
                KOZO=1.0E+1
             ENDIF
C
C
C            --------------------------
C            Compute the water abs coef
C            --------------------------
             KWAT=( COEF2(23,ILAY,I)*WPRED2( 1,ILAY) ) +
     $            ( COEF2(24,ILAY,I)*WPRED2( 2,ILAY) ) +
     $            ( COEF2(25,ILAY,I)*WPRED2( 3,ILAY) ) +
     $            ( COEF2(26,ILAY,I)*WPRED2( 4,ILAY) ) +
     $            ( COEF2(27,ILAY,I)*WPRED2( 5,ILAY) ) +
     $            ( COEF2(28,ILAY,I)*WPRED2( 6,ILAY) ) +
     $            ( COEF2(29,ILAY,I)*WPRED2( 7,ILAY) ) +
     $            ( COEF2(30,ILAY,I)*WPRED2( 8,ILAY) ) +
     $            ( COEF2(31,ILAY,I)*WPRED2( 9,ILAY) ) +
     $            ( COEF2(32,ILAY,I)*WPRED2(10,ILAY) ) +
     $            ( COEF2(33,ILAY,I)*WPRED2(11,ILAY) )
C
             IF (KWAT .LT. 0.0E+0) THEN
                KWAT=0.0E+0
             ELSEIF( KWAT .GT. 1.0E+1) THEN
                KWAT=1.0E+1
             ENDIF
C
C            ----------------------------------
C            Calc the total layer transmittance
C            ----------------------------------
c
ccccc
c This block is usually commented out and is only uncommented for
c testing purposes.
c
c           kcon=0.0E+0
c           kfix=0.0E+0
c           kozo=0.0E+0
c           kwat=0.0E+0
ccccc
C
C            Calc the total layer effective optical depth
             KLAYER=KCON + KFIX + KOZO + KWAT

C            Adjust the optical depth of the bottom layer
             IF (ILAY .EQ. NLAY) KLAYER=BLMULT*KLAYER
C
             KZ=KZ + KLAYER
             TAU(ILAY,J)=QIKEXP(-KLAYER)
C
          ENDDO
C         End loop on levels
C
C         Convert KZ to TAUZ
          TAUZ(J)=QIKEXP(-KZ)
C
        ENDIF
C       End of if-then for LCHNON "on"
C
       ENDDO
C      End loops on channel number (frequency)
C
       RETURN
       END
