C=======================================================================
C=======================================================================
C
C    University of Maryland Baltimore County [UMBC]
C
C    HIS
C
C    CALT3 (for FOW sun mfbw)
C
!F77====================================================================


!ROUTINE NAME:
C    CALT3


!ABSTRACT:
C    Calculate the transmittance TAU using the predictors and the fast
C    transmittance coefficients.


!CALL PROTOCOL:
C    CALT3( LTAU, INDCHN, NLAY, BLMULT, NCHN3, CLIST3, LCHNON, COEF3,
C       FIXMUL, CONPD3, FPRED3, WPRED3, OPRED3, TAU, TAUZ )


!INPUT PARAMETERS:
C    type      name    purpose                     units
C    --------  ------  --------------------------  ---------------------
C    LOGICAL   LTAU    calc all layer trans?       none
C    INT arr   INDCHN  channel indices             none
C    INTEGER   NLAY    number of layers to bottom  none
C    REAL      BLMULT  bottom layer opt depth mult none
C    INTEGER   NCHN3   number of channels          none
C    INT arr   CLIST3  channel list                none
C    LOG arr   LCHNON  chan calc on/off switch     none
C    REAL arr  COEF3   fast trans coefs            various
C    REAL arr  FIXMUL  fixed amount mult (~1.0)    none
C    REAL arr  CONPD3  H2O continuum preds         various
C    REAL arr  FPRED3  fixed gases preds           various
C    REAL arr  WPRED3  water predictors            various
C    REAL arr  OPRED3  ozone predictors            various


!OUTPUT PARAMETERS:
C    type      name    purpose                     units
C    --------  ------  --------------------------  ---------------------
C    REAL arr  TAU     effective layer trans       none
C    REAL arr  TAUZ    layer-to-space trans        none


!INPUT/OUTPUT PARAMETERS:
C    none


!RETURN VALUES:
C    none


!PARENT(S):
C    USEFAST


!ROUTINES CALLED:
C    none


!FILES ACCESSED:
C    incFTC.f : include file of parameter statements accessed during
C       compilation only.


!COMMON BLOCKS
C    none


!DESCRIPTION:
C    August 1998 version of the 65 layer NAST Fast Transmittance
C    Code by Howard Motteler/Larrabee Strow/Scott Hannon.
C
C    The fast trans coefficents and predictors are multiplied
C    together and summed to calculate the effecttive layer
C    transmittances. Fixed, water, and ozone transmittances are each
C    checked individually to be sure they give 0 < tau < 1.
C
C    ===================================================================
C    Loops downward over all the layers for each of the NCHN3 channels
C    to compute the layer transmittances TAU.
C
C    The water continuum absorption coefficient is:
C       k_con = the sum i=1 to 4 of { COEF(i)*CONPRD(i) }
C
C    The layer effective fixed gas absorption coefficient is:
C       k_fixed = the sum i=1 to 8 of { COEF(4+i)*FPRED(i) }
C
C    The layer effective ozone absorption coefficient is:
C       k_ozone = the sum i=1 to 3 of { COEF(4+8+i)*OPRED(i) }
C
C    The layer effective water lines absorption coefficient is:
C       k_water = the sum i=1 to 13 of { COEF(4+8+3+i)*WPRED(i) }
C
C    where
C      "COEF" are the fast transmittance coefficients COEF3
C      "CONPRD" are the water continuum predictors CONPRD
C      "FPRED" are the fixed gases predictors FPRED3
C      "WPRED" are the water lines predictors WPRED3
C      "OPRED" are the ozone predictors OPRED3
C
C    The total layer effective transmittance is:
C       TAU = exp( -[ k_con + k_fixed + k_water + k_ozone ])
C
C    To help speed up the exponential calculations, we use our own
C    "EXP" replacement function called QIKEXP which uses just the
C    first few series expansion terms for exp(x) if x is suitably small.
C    ===================================================================


!ALGORITHM REFERENCES:
C    none


!KNOWN BUGS AND LIMITATIONS:
C    none


!ROUTINE HISTORY:
C    Date        Programmer     Comments
C    ----------- -------------- ----------------------------------------
C     7 Aug 1998 Scott Hannon   Created based on Jul98 HIS version
C    10 Sep 1998 Scott Hannon   Added LCHNON


!END====================================================================

C      =================================================================
       SUBROUTINE CALT3 ( LTAU, INDCHN, NLAY, BLMULT, NCHN3, CLIST3,
     $    LCHNON, COEF3, FIXMUL, CONPD3, FPRED3, WPRED3, OPRED3,
     $    TAU, TAUZ )
C      =================================================================

C-----------------------------------------------------------------------
C      IMPLICIT NONE
C-----------------------------------------------------------------------
       IMPLICIT NONE


C-----------------------------------------------------------------------
C      INCLUDE FILES
C-----------------------------------------------------------------------
#include "incFTC.f"


C-----------------------------------------------------------------------
C      EXTERNAL FUNCTIONS
C-----------------------------------------------------------------------
C      QIKEXP


C-----------------------------------------------------------------------
C      ARGUMENTS
C-----------------------------------------------------------------------
C      Input
       LOGICAL   LTAU
       INTEGER INDCHN(MXCHAN)
       INTEGER   NLAY
       REAL BLMULT
       INTEGER  NCHN3
       INTEGER CLIST3(MXCHN3)
       LOGICAL LCHNON(MXCHAN)
       REAL  COEF3(N3COEF,MAXLAY,MXCHN3)
       REAL FIXMUL(MAXLAY)
       REAL CONPD3( N3CON,MAXLAY)
       REAL FPRED3( N3FIX,MAXLAY)
       REAL WPRED3( N3H2O,MAXLAY)
       REAL OPRED3(  N3O3,MAXLAY)
C
C      Output
       REAL    TAU(MAXLAY,MXCHAN)
       REAL   TAUZ(MXCHAN)


C-----------------------------------------------------------------------
C      LOCAL VARIABLES
C-----------------------------------------------------------------------
       INTEGER      I
       INTEGER   ILAY
       INTEGER      J
       REAL   KCON
       REAL   KFIX
       REAL KLAYER
       REAL   KOZO
       REAL   KWAT
       REAL     KZ
C
C      for function QIKEXP
       REAL QIKEXP


C-----------------------------------------------------------------------
C      SAVE STATEMENTS
C-----------------------------------------------------------------------
C      none


C***********************************************************************
C***********************************************************************
C                    EXECUTABLE CODE
C***********************************************************************
C***********************************************************************
C
C      ---------------------------
C      Loop on channel (frequency)
C      ---------------------------
       DO I=1,NCHN3
C
C       Only do calc if channel is turned "on"
        IF (LCHNON(CLIST3(I))) THEN
C
C         Index for TAU
          J=INDCHN( CLIST3(I) )
C
C         Initialize the layer-to-space optical depth
          KZ=0.0E+0
C
C         ------------------------------
C         Loop on layers (top to ground)
C         ------------------------------
          DO ILAY=1,NLAY
C
C
C            ---------------------------
C            Compute the water continuum
C            ---------------------------
             KCON=( COEF3(1,ILAY,I)*CONPD3(1,ILAY) ) +
     $            ( COEF3(2,ILAY,I)*CONPD3(2,ILAY) ) +
     $            ( COEF3(3,ILAY,I)*CONPD3(3,ILAY) ) +
     $            ( COEF3(4,ILAY,I)*CONPD3(4,ILAY) )
C
             IF (KCON .LT. 0.0E+0) THEN
                KCON=0.0E+0
             ELSEIF (KCON .GT. 1.0E+1) THEN
                KCON=1.0E+1
             ENDIF
C
C
C            -----------------------------
C            Calc the fixed gases abs coef
C            -----------------------------
             KFIX=( COEF3( 5,ILAY,I)*FPRED3( 1,ILAY) ) +
     $            ( COEF3( 6,ILAY,I)*FPRED3( 2,ILAY) ) +
     $            ( COEF3( 7,ILAY,I)*FPRED3( 3,ILAY) ) +
     $            ( COEF3( 8,ILAY,I)*FPRED3( 4,ILAY) ) +
     $            ( COEF3( 9,ILAY,I)*FPRED3( 5,ILAY) ) +
     $            ( COEF3(10,ILAY,I)*FPRED3( 6,ILAY) ) +
     $            ( COEF3(11,ILAY,I)*FPRED3( 7,ILAY) ) +
     $            ( COEF3(12,ILAY,I)*FPRED3( 8,ILAY) )
C
cccc
c removed for comparing with unfit data
c             KFIX=KFIX*FIXMUL(L)
cccc
C
             IF (KFIX .LT. 0.0E+0) THEN
                KFIX=0.0E+0
             ELSEIF (KFIX .GT. 1.0E+1) THEN
                KFIX=1.0E+1
             ENDIF
C
C
C            --------------------------
C            Compute the ozone abs coef
C            --------------------------
             KOZO=( COEF3(13,ILAY,I)*OPRED3( 1,ILAY) ) +
     $            ( COEF3(14,ILAY,I)*OPRED3( 2,ILAY) ) +
     $            ( COEF3(15,ILAY,I)*OPRED3( 3,ILAY) )
C
             IF (KOZO .LT. 0.0E+0) THEN
                KOZO=0.0E+0
             ELSEIF( KOZO .GT. 1.0E+1) THEN
                KOZO=1.0E+1
             ENDIF
C
C
C            --------------------------
C            Compute the water abs coef
C            --------------------------
             KWAT=( COEF3(16,ILAY,I)*WPRED3( 1,ILAY) ) +
     $            ( COEF3(17,ILAY,I)*WPRED3( 2,ILAY) ) +
     $            ( COEF3(18,ILAY,I)*WPRED3( 3,ILAY) ) +
     $            ( COEF3(19,ILAY,I)*WPRED3( 4,ILAY) ) +
     $            ( COEF3(20,ILAY,I)*WPRED3( 5,ILAY) ) +
     $            ( COEF3(21,ILAY,I)*WPRED3( 6,ILAY) ) +
     $            ( COEF3(22,ILAY,I)*WPRED3( 7,ILAY) ) +
     $            ( COEF3(23,ILAY,I)*WPRED3( 8,ILAY) ) +
     $            ( COEF3(24,ILAY,I)*WPRED3( 9,ILAY) ) +
     $            ( COEF3(25,ILAY,I)*WPRED3(10,ILAY) ) +
     $            ( COEF3(26,ILAY,I)*WPRED3(11,ILAY) ) +
     $            ( COEF3(27,ILAY,I)*WPRED3(12,ILAY) ) +
     $            ( COEF3(28,ILAY,I)*WPRED3(13,ILAY) )
C
             IF (KWAT .LT. 0.0E+0) THEN
                KWAT=0.0E+0
             ELSEIF (KWAT .GT. 1.0E+1) THEN
                KWAT=1.0E+1
             ENDIF
C
C
C            ----------------------------------
C            Calc the total layer transmittance
C            ----------------------------------
c
ccccc
c This block is usually commented out and is only uncommented for
c testing purposes.
c
c           kcon=0.0E+0
c           kfix=0.0E+0
c           kozo=0.0E+0
c           kwat=0.0E+0
ccccc
C
C            Calc the total layer effective optical depth
             KLAYER=KCON + KFIX + KOZO + KWAT
C
C            Adjust the optical depth of the bottom layer
             IF (ILAY .EQ. NLAY) KLAYER=BLMULT*KLAYER
C
             KZ=KZ + KLAYER
             IF (LTAU) TAU(ILAY,J)=QIKEXP(-KLAYER)
C
          ENDDO
C         End loop on levels
C
C         Convert KZ to TAUZ
          TAUZ(J)=QIKEXP(-KZ)
C
        ENDIF
C       End of if-then for LCHNON "on"
C
       ENDDO
C      End loops on channel number (frequency)
C
       RETURN
       END
