C=======================================================================
C=======================================================================
C
C    University of Maryland Baltimore County [UMBC]
C
C    NAST
C
C    CALT4 (for set4 = FCOW)
C
!F77====================================================================


!ROUTINE NAME:
C    CALT4


!ABSTRACT:
C    Calculate the transmittance TAU using the predictors and the fast
C    transmittance coefficients.


!CALL PROTOCOL:
C    CALT4( LTAU, INDCHN, NLAY, BMULT, NCHN4, CLIST4, LCHNON, COEF4,
C       FIXMUL, CONPD4, FPRED4, WPRED4, OPRED4, CPRED4, TAU, TAUZ )


!INPUT PARAMETERS:
C    type      name    purpose                     units
C    --------  ------  --------------------------  ---------------------
C    LOGICAL   LTAU    calc all layer trans?       none
C    INT arr   INDCHN  channel indices             none
C    INTEGER   NLAY    number of layers to bottom  none
C    REAL      BLMULT  bottom layer opt depth mult none
C    INTEGER   NCHN4   number of channels          none
C    INT arr   CLIST4  channel list                none
C    LOG arr   LCHNON  chan calc on/off switch     none
C    REAL arr  COEF4   fast trans coefs            various
C    REAL arr  FIXMUL  fixed amount mult (~1.0)    none
C    REAL arr  CONPD4  H2O continuum preds         various
C    REAL arr  FPRED4  fixed gases preds           various
C    REAL arr  WPRED4  water predictors            various
C    REAL arr  OPRED4  ozone predictors            various
C    REAL arr  CPRED4  CO    predictors            various


!OUTPUT PARAMETERS:
C    type      name    purpose                     units
C    --------  ------  --------------------------  ---------------------
C    REAL arr  TAU     effective layer trans       none
C    REAL arr  TAUZ    layer-to-space trans        none


!INPUT/OUTPUT PARAMETERS:
C    none


!RETURN VALUES:
C    none


!PARENT(S):
C    USEFAST


!ROUTINES CALLED:
C    none


!FILES ACCESSED:
C    incFTC.f : include file of parameter statements accessed during
C       compilation only.


!COMMON BLOCKS
C    none


!DESCRIPTION:
C    August 1998 version of the 65 layer NAST Fast Transmittance
C    Code by Howard Motteler/Larrabee Strow/Scott Hannon.
C
C    The fast trans coefficents and predictors are multiplied
C    together and summed to calculate the effecttive layer
C    transmittances. Fixed, water, and ozone transmittances are each
C    checked individually to be sure they give 0 < tau < 1.
C
C    ===================================================================
C    Loops downward over all the layers for each of the NCHN4 channels
C    to compute the layer transmittances TAU.
C
C    The water continuum absorption coefficient is:
C       k_con = the sum i=1 to 4 of { COEF(i)*CONPRD(i) }
C
C    The layer effective fixed gas absorption coefficient is:
C       k_fixed = the sum i=1 to 11 of { COEF(4+i)*FPRED(i) }
C
C    The layer effective ozone absorption coefficient is:
C       k_co = the sum i=1 to 11 of { COEF(4+11+i)*CPRED(i) }
C
C    The layer effective ozone absorption coefficient is:
C       k_ozone = the sum i=1 to 9 of { COEF(4+11+11+i)*OPRED(i) }
C
C    The layer effective water lines absorption coefficient is:
C       k_water = the sum i=1 to 13 of { COEF(4+11+11+9+i)*WPRED(i) }
C
C    where
C      "COEF" are the fast transmittance coefficients COEF4
C      "CONPRD" are the water continuum predictors CONPRD
C      "FPRED" are the fixed gases predictors FPRED4
C      "CPRED" are the CO predictors CPRED4
C      "OPRED" are the ozone predictors OPRED4
C      "WPRED" are the water lines predictors WPRED4
C
C    The total layer effective transmittance is:
C       TAU = exp( -[ k_con + k_fixed + k_co + k_ozone + k_water ])
C
C    To help speed up the exponential calculations, we use our own
C    "EXP" replacement function called QIKEXP which uses just the
C    first few series expansion terms for exp(x) if x is suitably small.
C    ===================================================================


!ALGORITHM REFERENCES:
C    none


!KNOWN BUGS AND LIMITATIONS:
C    none


!ROUTINE HISTORY:
C    Date        Programmer     Comments
C    ----------- -------------- ----------------------------------------
C     7 Aug 1998 Scott Hannon   Created based on Jul98 HIS version
C    10 Sep 1998 Scott Hannon   Added LCHNON


!END====================================================================

C      =================================================================
       SUBROUTINE CALT4 ( LTAU, INDCHN, NLAY, BLMULT, NCHN4, CLIST4,
     $    LCHNON, COEF4, FIXMUL, CONPD4, FPRED4, WPRED4, OPRED4, CPRED4,
     $    TAU, TAUZ )
C      =================================================================

C-----------------------------------------------------------------------
C      IMPLICIT NONE
C-----------------------------------------------------------------------
       IMPLICIT NONE


C-----------------------------------------------------------------------
C      INCLUDE FILES
C-----------------------------------------------------------------------
#include "incFTC.f"


C-----------------------------------------------------------------------
C      EXTERNAL FUNCTIONS
C-----------------------------------------------------------------------
C      QIKEXP


C-----------------------------------------------------------------------
C      ARGUMENTS
C-----------------------------------------------------------------------
C      Input
       LOGICAL   LTAU
       INTEGER INDCHN(MXCHAN)
       INTEGER   NLAY
       REAL BLMULT
       INTEGER  NCHN4
       INTEGER CLIST4(MXCHN4)
       LOGICAL LCHNON(MXCHAN)
       REAL  COEF4(N4COEF,MAXLAY,MXCHN4)
       REAL FIXMUL(MAXLAY)
       REAL CONPD4( N4CON,MAXLAY)
       REAL FPRED4( N4FIX,MAXLAY)
       REAL WPRED4( N4H2O,MAXLAY)
       REAL OPRED4(  N4O3,MAXLAY)
       REAL CPRED4(  N4CO,MAXLAY)
C
C      Output
       REAL    TAU(MAXLAY,MXCHAN)
       REAL   TAUZ(MXCHAN)


C-----------------------------------------------------------------------
C      LOCAL VARIABLES
C-----------------------------------------------------------------------
       INTEGER      I
       INTEGER      J
       INTEGER   ILAY
       REAL   KCON
       REAL   KFIX
       REAL   KWAT
       REAL   KOZO
       REAL    KCO
       REAL     KZ
       REAL KLAYER
C
C      for function QIKEXP
       REAL QIKEXP


      integer ic

C-----------------------------------------------------------------------
C      SAVE STATEMENTS
C-----------------------------------------------------------------------
C      none


C***********************************************************************
C***********************************************************************
C                    EXECUTABLE CODE
C***********************************************************************
C***********************************************************************
C
C      ---------------------------
C      Loop on channel (frequency)
C      ---------------------------
       DO I=1,NCHN4
C
C       Only do calc if channel is turned "on"
        IF (LCHNON(CLIST4(I))) THEN
C
C         Index for TAU
          J=INDCHN( CLIST4(I) )
C
C         Initialize the layer-to-space optical depth
          KZ=0.0E+0
C
C         ------------------------------
C         Loop on layers (top to ground)
C         ------------------------------
          DO ILAY=1,NLAY
C
C
C            ---------------------------
C            Compute the water continuum
C            ---------------------------
             KCON=( COEF4(1,ILAY,I)*CONPD4(1,ILAY) ) +
     $            ( COEF4(2,ILAY,I)*CONPD4(2,ILAY) ) +
     $            ( COEF4(3,ILAY,I)*CONPD4(3,ILAY) ) +
     $            ( COEF4(4,ILAY,I)*CONPD4(4,ILAY) )
C
             IF (KCON .LT. 0.0E+0) THEN
                KCON=0.0E+0
             ELSEIF (KCON .GT. 1.0E+1) THEN
                KCON=1.0E+1
             ENDIF
C
C
C            -----------------------------
C            Calc the fixed gases abs coef
C            -----------------------------
             KFIX=( COEF4( 5,ILAY,I)*FPRED4( 1,ILAY) ) +
     $            ( COEF4( 6,ILAY,I)*FPRED4( 2,ILAY) ) +
     $            ( COEF4( 7,ILAY,I)*FPRED4( 3,ILAY) ) +
     $            ( COEF4( 8,ILAY,I)*FPRED4( 4,ILAY) ) +
     $            ( COEF4( 9,ILAY,I)*FPRED4( 5,ILAY) ) +
     $            ( COEF4(10,ILAY,I)*FPRED4( 6,ILAY) ) +
     $            ( COEF4(11,ILAY,I)*FPRED4( 7,ILAY) ) +
     $            ( COEF4(12,ILAY,I)*FPRED4( 8,ILAY) ) +
     $            ( COEF4(13,ILAY,I)*FPRED4( 9,ILAY) ) +
     $            ( COEF4(14,ILAY,I)*FPRED4(10,ILAY) ) +
     $            ( COEF4(15,ILAY,I)*FPRED4(11,ILAY) )
C
cccc
c removed for comparing with unfit data
c             KFIX=KFIX*FIXMUL(L)
cccc
C
             IF (KFIX .LT. 0.0E+0) THEN
                KFIX=0.0E+0
             ELSEIF (KFIX .GT. 1.0E+1) THEN
                KFIX=1.0E+1
             ENDIF
C
C
C            -----------------------
C            Compute the CO abs coef
C            -----------------------
             KCO=( COEF4(16,ILAY,I)*CPRED4( 1,ILAY) ) +
     $           ( COEF4(17,ILAY,I)*CPRED4( 2,ILAY) ) +
     $           ( COEF4(18,ILAY,I)*CPRED4( 3,ILAY) ) +
     $           ( COEF4(19,ILAY,I)*CPRED4( 4,ILAY) ) +
     $           ( COEF4(20,ILAY,I)*CPRED4( 5,ILAY) ) +
     $           ( COEF4(21,ILAY,I)*CPRED4( 6,ILAY) ) +
     $           ( COEF4(22,ILAY,I)*CPRED4( 7,ILAY) ) +
     $           ( COEF4(23,ILAY,I)*CPRED4( 8,ILAY) ) +
     $           ( COEF4(24,ILAY,I)*CPRED4( 9,ILAY) ) +
     $           ( COEF4(25,ILAY,I)*CPRED4(10,ILAY) ) +
     $           ( COEF4(26,ILAY,I)*CPRED4(11,ILAY) )
C
             IF (KCO .LT. 0.0E+0) THEN
                KCO=0.0E+0
             ELSEIF( KCO .GT. 1.0E+1) THEN
                KCO=1.0E+1
             ENDIF
C
C
C            --------------------------
C            Compute the ozone abs coef
C            --------------------------
             KOZO=( COEF4(27,ILAY,I)*OPRED4( 1,ILAY) ) +
     $            ( COEF4(28,ILAY,I)*OPRED4( 2,ILAY) ) +
     $            ( COEF4(29,ILAY,I)*OPRED4( 3,ILAY) ) +
     $            ( COEF4(30,ILAY,I)*OPRED4( 4,ILAY) ) +
     $            ( COEF4(31,ILAY,I)*OPRED4( 5,ILAY) ) +
     $            ( COEF4(32,ILAY,I)*OPRED4( 6,ILAY) ) +
     $            ( COEF4(33,ILAY,I)*OPRED4( 7,ILAY) ) +
     $            ( COEF4(34,ILAY,I)*OPRED4( 8,ILAY) ) +
     $            ( COEF4(35,ILAY,I)*OPRED4( 9,ILAY) )
C
             IF (KOZO .LT. 0.0E+0) THEN
                KOZO=0.0E+0
             ELSEIF( KOZO .GT. 1.0E+1) THEN
                KOZO=1.0E+1
             ENDIF
C
C
C            --------------------------
C            Compute the water abs coef
C            --------------------------
             KWAT=( COEF4(36,ILAY,I)*WPRED4( 1,ILAY) ) +
     $            ( COEF4(37,ILAY,I)*WPRED4( 2,ILAY) ) +
     $            ( COEF4(38,ILAY,I)*WPRED4( 3,ILAY) ) +
     $            ( COEF4(39,ILAY,I)*WPRED4( 4,ILAY) ) +
     $            ( COEF4(40,ILAY,I)*WPRED4( 5,ILAY) ) +
     $            ( COEF4(41,ILAY,I)*WPRED4( 6,ILAY) ) +
     $            ( COEF4(42,ILAY,I)*WPRED4( 7,ILAY) ) +
     $            ( COEF4(43,ILAY,I)*WPRED4( 8,ILAY) ) +
     $            ( COEF4(44,ILAY,I)*WPRED4( 9,ILAY) ) +
     $            ( COEF4(45,ILAY,I)*WPRED4(10,ILAY) ) +
     $            ( COEF4(46,ILAY,I)*WPRED4(11,ILAY) ) +
     $            ( COEF4(47,ILAY,I)*WPRED4(12,ILAY) ) +
     $            ( COEF4(48,ILAY,I)*WPRED4(13,ILAY) )
C
             IF (KWAT .LT. 0.0E+0) THEN
                KWAT=0.0E+0
             ELSEIF (KWAT .GT. 1.0E+1) THEN
                KWAT=1.0E+1
             ENDIF
C
C
C            ----------------------------------
C            Calc the total layer transmittance
C            ----------------------------------
c
ccccc
c This block is usually commented out and is only uncommented for
c testing purposes.
c
c           kcon=0.0E+0
c           kfix=0.0E+0
c            kco=0.0E+0
c           kozo=0.0E+0
c           kwat=0.0E+0
ccccc
C
C            Calc the total layer effective optical depth
             KLAYER=KCON + KFIX + KCO + KOZO + KWAT 
C
C            Adjust the optical depth of the bottom layer
             IF (ILAY .EQ. NLAY) KLAYER=BLMULT*KLAYER
C
             KZ=KZ + KLAYER
             IF (LTAU) TAU(ILAY,J)=QIKEXP(-KLAYER)
C
          ENDDO
C         End loop on levels
C
C         Convert KZ to TAUZ
          TAUZ(J)=QIKEXP(-KZ)
C
        ENDIF
C       End of if-then for LCHNON "on"
C
       ENDDO
C      End loops on channel number (frequency)
C
       RETURN
       END
