C=======================================================================
C=======================================================================
C
C    University of Maryland Baltimore County (UMBC)
C
C    NAST
C
C    USEFAST
C
!F77====================================================================


!ROUTINE NAME:
C    USEFAST


!ABSTRACT:
C    Demo program to compute radiances using the August 1998 version
C    of the 65 layer NAST Fast Transmittance Code by Howard Motteler/
C    Larrabee Strow/Scott Hannon.


!CALL PROTOCOL
C    none (main program)


!INPUT PARAMETERS:
C    none


!OUTPUT PARAMETERS:
C    none


!INPUT/OUTPUT PARAMETERS:
C    none


!RETURN VALUES:
C    none


!PARENT(S)
C    none


!ROUTINES CALLED:
C    CALPAR : calculate a profile's predictors
C    CALRAD : calc radiance
C    CALT1  : calc effective layer trans for set1 (FWO)
C    CALT2  : calc effective layer trans for set2 (FOW)
C    CALT3  : calc effective layer trans for set3 (FMW)
C    CALT4  : calc effective layer trans for set4 (FCOW)
C    CALT5  : calc effective layer trans for set5 (FWO bfsw)
C    FAKETZ : calc a "fake" (rough approx) surface-to-space trans
C    RDLIST : read the channel list
C    RDCOEF : read the fast transmittance coefficients
C    RDPROF : read profile data
C    RDSUN  : read the solar radiance datafile
C    SUNPAR : calc a profile's predictors for sets4-7 (sun channels)


!FILES ACCESSED:
C    incFTC.f : include file of parameter statements accessed during
C       compilation only.
C    unit IOUN: used by routines RDLIST, RDCOEF, and RDPROF.
C    unit 6: USEFAST text messages to the screen
C    unit 5: USEFAST user input instructions, etc
C    unit 10: USEFAST output radiance, text file(s)


!COMMON BLOCKS
C    none


!DESCRIPTION:
C    August 1998 version of the 65 layer NAST Fast Transmittance
C    Code by Larrabee Strow/Scott Hannon/Howard Motteler.
C
C    Compute a radiance for a given profile using the 65 layer NAST
C    rapid transmittance algorithm.
C
C    This is the main program, and consists primarily of calls to
C    the external routines to do most of the computing.


!ALGORITHM REFERENCES:
C    none


!KNOWN BUGS AND LIMITATIONS:
C    This program is only intended as a demo of the fast model.


!ROUTINE HISTORY:
C    Date       Programmer     Comments
C    ---------  -------------- -----------------------------------------
C     7 Aug 98  Scott Hannon   Created based on Mar98 AIRS version
C    26 Aug 98  Scott Hannon   Added LBOT to CALPAR call; rename TBOT to
C                              TSURF; calc fractional bottom air layer
C                              temperature and put into TEMP(LBOT).
C    10 Sep 98  Scott Hannon   Added LCHNON and LSUNON


!END====================================================================

C      =================================================================
       PROGRAM USEFAST
C      =================================================================


C-----------------------------------------------------------------------
C      IMPLICIT NONE
C-----------------------------------------------------------------------
       IMPLICIT NONE


C-----------------------------------------------------------------------
C      INCLUDE FILES
C-----------------------------------------------------------------------
#include "incFTC.f"


C-----------------------------------------------------------------------
C      EXTERNAL FUNCTIONS
C-----------------------------------------------------------------------
       REAL SACONV


C-----------------------------------------------------------------------
C      ARGUMENTS
C-----------------------------------------------------------------------
C      none (main program)


C-----------------------------------------------------------------------
C      LOCAL VARIABLES
C-----------------------------------------------------------------------
C
C      for RDLIST
       INTEGER   IOUN         ! I/O unit number
       INTEGER  NCHAN         ! # of selected channels
       INTEGER LSTCHN(MXCHAN) ! list of selected channels
       INTEGER INDCHN(MXCHAN) ! array indices for all channels
C
C      for RDCOEF             ! Info for selected channels only
       INTEGER SETCHN(MXCHAN) ! set # for each channel
       INTEGER  NCHN1         ! # of set1 channels
       INTEGER  NCHN2         ! # of set2 channels
       INTEGER  NCHN3         ! # of set3 channels
       INTEGER  NCHN4         ! # of set4 channels
       INTEGER CLIST1(MXCHN1) ! list of set1 channels
       INTEGER CLIST2(MXCHN2) ! list of set2 channels
       INTEGER CLIST3(MXCHN3) ! list of set3 channels
       INTEGER CLIST4(MXCHN4) ! list of set4 channels
       INTEGER LABOVE(MXCHAN) ! chan downwelling thermal layer above
       REAL  COEF1(N1COEF,MAXLAY,MXCHN1) ! coefs for set1 chans
       REAL  COEF2(N2COEF,MAXLAY,MXCHN2) ! coefs for set2 chans
       REAL  COEF3(N3COEF,MAXLAY,MXCHN3) ! coefs for set3 chans
       REAL  COEF4(N4COEF,MAXLAY,MXCHN4) ! coefs for set4 chans
       REAL   FREQ(MXCHAN)               ! chan center frequency
       REAL  COEFF(NFCOEF,MXCHAN)        ! coefs for chan "F" factor
C
C      for FAKETZ
       INTEGER  NFAKE         ! # of channels to do "fake" sun calc
       INTEGER  NREAL         ! # of channels to do "real" sun calc
       INTEGER INDFAK(MXCHAN) ! indices of channels to fake
C
C      for RDPROF; reference profile
       CHARACTER*80 RPFILE ! ref prof filename
       CHARACTER*40  RPNAM ! ref prof name/ID
       REAL   RALT(MAXLAY) ! ref prof layer altitude
       REAL    RDZ(MAXLAY) ! ref prof layer thickness
       REAL  RPRES(MAXLAY) ! ref prof layer average pressure
       REAL  RTEMP(MAXLAY) ! ref prof layer average temperature
       REAL RFAMNT(MAXLAY) ! ref prof layer "fixed" (CO2) amount
       REAL RWAMNT(MAXLAY) ! ref prof layer water (H2O) amount
       REAL ROAMNT(MAXLAY) ! ref prof layer ozone (O3) amount
       REAL RCAMNT(MAXLAY) ! ref prof layer carbon monoxide (CO) amount
C
C      for RDPROF; profile to calculate
       CHARACTER*80  PFILE ! prof filename
       CHARACTER*40   PNAM ! prof name/ID
       REAL    ALT(MAXLAY) ! prof layer altitudes
       REAL     DZ(MAXLAY) ! prof layer thicknesses
       REAL   PRES(MAXLAY) ! prof layer average pressure
       REAL   TEMP(MAXLAY) ! prof layer average temperature
       REAL  FAMNT(MAXLAY) ! prof layer "fixed" (CO2) amount
       REAL  WAMNT(MAXLAY) ! prof layer water (H2O) amount
       REAL  OAMNT(MAXLAY) ! prof layer ozone (O3) amount
       REAL  CAMNT(MAXLAY) ! prof layer carbon monoxide (CO) amount
C
C      for CALPAR
       REAL SECANG(MAXLAY)        ! local path angle secant
       REAL FIXMUL(MAXLAY)        ! "fixed" amount multiplier (~1)
       REAL CONPRD( N1CON,MAXLAY) ! water continuum predictors
       REAL FPRED1( N1FIX,MAXLAY) ! set1 "fixed" predictors
       REAL FPRED2( N2FIX,MAXLAY) ! set2 "fixed" predictors
       REAL FPRED3( N3FIX,MAXLAY) ! set3 "fixed" predictors
       REAL FPRED4( N4FIX,MAXLAY) ! set4 "fixed" predictors
       REAL WPRED1( N1H2O,MAXLAY) ! set1 water predictors
       REAL WPRED2( N2H2O,MAXLAY) ! set2 water predictors
       REAL WPRED3( N3H2O,MAXLAY) ! set3 water predictors
       REAL WPRED4( N4H2O,MAXLAY) ! set4 water predictors
       REAL OPRED1(  N1O3,MAXLAY) ! set1 ozone predictors
       REAL OPRED2(  N2O3,MAXLAY) ! set2 ozone predictors
       REAL OPRED3(  N3O3,MAXLAY) ! set3 ozone predictors
       REAL OPRED4(  N4O3,MAXLAY) ! set4 ozone predictors
       REAL CPRED4(  N4CO,MAXLAY) ! set4 carbon monoxide predictors
C
C      for CALT
       INTEGER   LBOT             ! bottom layer number
       REAL BLMULT                ! bottom layer multiplier
       REAL    TAU(MAXLAY,MXCHAN) ! chan layer effective trans
       REAL   TAUZ(MXCHAN)        ! chan surface-to-space trans
       LOGICAL   LTAU             ! calc all layer transmittances?
       LOGICAL LCHNON(MXCHAN)     ! channel calc on/off switch
C
C      for CALRAD
       REAL  TSURF         ! surface temperature
       REAL   EBOT(MXCHAN) ! chan surface emissivity
       REAL RHOSUN(MXCHAN) ! chan reflectivity for sun
       REAL RHOTHR(MXCHAN) ! chan reflectivity for downwelling thermal
       REAL    RAD(MXCHAN) ! chan radiance
       REAL     BT(MXCHAN) ! chan brightness temperature
C
C      for RDSUN
       REAL   HSUN(MXCHAN) ! sun radiance (direct from sun)
C
C      Other variables for the sun
       REAL SUNANG            ! solar zenith angle (at 0 altitude)
       REAL SZALAY            ! solar zenith angle in some layer
       REAL SUNCOS            ! cosine of sun zenith angle
       REAL SUNSEC            ! secant of sun zenith angle = 1/suncos
       REAL SECSUN(MAXLAY)    ! secant of effective sun local path angle
       REAL DISTES            ! distance of Earth from the sun
       REAL TAUZSN(MXCHAN)    ! chan eff sun angle surf-to-plane trans
       LOGICAL DOSUN          ! do sun calc?
       LOGICAL LSUNON(MXCHAN) ! channel on/off switch for sun calc
C
C      instrument viewing angle
       REAL    SVA         ! instrument viewing angle (degrees)
C
C      used locally only
       INTEGER      I      ! loop counter
       INTEGER      L      ! loop counter
       INTEGER NBND12      ! total number of channels in bands 1 and 2
       REAL OFFSET         ! surface - air temperature offset
       REAL   CONV         ! degrees to radians conversion factor
       REAL TSFAIR
       CHARACTER*1  YNANS  ! y/n (yes or no)
       CHARACTER*80  CLINE ! character line


C-----------------------------------------------------------------------
C      SAVE STATEMENTS
C-----------------------------------------------------------------------
C      none


C***********************************************************************
C***********************************************************************
C                    EXECUTABLE CODE
C***********************************************************************
C***********************************************************************
C
C      CONV = pi/180 = degrees to radians conversion factor
       CONV=1.7453292E-02
C
C      --------------------------
C      Assign the I/O unit number
C      --------------------------
       IOUN=11
C
C      -------------------------------------
C      Read in the lists of desired channels
C      -------------------------------------
C
       CALL RDLIST( IOUN, NCHAN, LSTCHN, INDCHN )
C
C      ------------------------
C      Read the coef data files
C      ------------------------
C
       CALL RDCOEF( IOUN, NCHAN, INDCHN, SETCHN,
     $     NCHN1,  NCHN2,  NCHN3,  NCHN4,
     $    CLIST1, CLIST2, CLIST3, CLIST4,
     $     COEF1,  COEF2,  COEF3,  COEF4,
     $     COEFF, LABOVE, FREQ )
C
C      ---------------------------------------------
C      All channels from bands 1 and 2 are to use a
C      fake effective sun angle layer-to-space trans
C      ---------------------------------------------
       NBND12=MXBND1 + MXBND2
       NFAKE=0
C
       DO I=1,NCHAN
          LCHNON( LSTCHN(I) )=.TRUE.
          IF (LSTCHN(I) .LE. NBND12) THEN
             NFAKE=NFAKE + 1
             INDFAK(NFAKE)=INDCHN( LSTCHN(I) )
             LSUNON( LSTCHN(I) )=.FALSE.
          ELSE
             LSUNON( LSTCHN(I) )=.TRUE.
          ENDIF
       ENDDO
C
       NREAL=NCHAN - NFAKE
C
C
C      --------------------------
C      Read in the solar radiance
C      --------------------------
C
       CALL RDSUN(IOUN, INDCHN, HSUN)
C
       DISTES=1.496E+11
C
C      -----------------------------
C      Read in the reference profile
C      -----------------------------
       WRITE(6,2010)
 2010  FORMAT('Enter name of reference profile')
       READ(5,9000) RPFILE
 9000  FORMAT(A80)
C
       CALL RDPROF(IOUN, RPFILE, RPNAM, RALT, RDZ, RPRES, RTEMP,
     $    RFAMNT, RWAMNT, ROAMNT, RCAMNT)
C
C      ------------------
C      Loop over profiles
C      ------------------
 20    WRITE(6,2020)
 2020  FORMAT('Enter name of profile to calcuate')
       READ(5,9000) PFILE
C
       CALL RDPROF(IOUN, PFILE, PNAM, ALT, DZ, PRES, TEMP,
     $    FAMNT, WAMNT, OAMNT, CAMNT)
C
C      -----------------------
C      Assign the bottom layer
C      -----------------------
       WRITE(6,2060)
 2060  FORMAT('Enter the bottom layer number (integer 1-65) and',
     $ ' multiplier (real ~1)')
       READ(5,*) LBOT, BLMULT
C
C      Calc fractional bottom layer temperature
       TEMP(LBOT)=TEMP(LBOT-1) + BLMULT*( TEMP(LBOT) - TEMP(LBOT-1) )
C
C      ------------------------
C      Loop over viewing angles
C      ------------------------
 30    WRITE(6,2030)
 2030  FORMAT('Enter the viewing angle (0-65 degrees)')
       READ(5,*) SVA
C
       IF (SVA .GT. 6.501E+1) THEN
          SVA=6.5E+1
       ELSEIF (SVA .LT. 0.0E+0) THEN
          SVA=0.0E+0
       ENDIF
C
C      Assign the viewing angle for all layers
       SECANG(1)=1.0E+0/COS( CONV*SVA )
       DO L=1,LBOT
          SECANG(L)=SECANG(1)
       ENDDO
C
C      Get the sun angle
 2035  FORMAT('Enter the sun angle (0-80 degrees, 0> no sun)')
       READ(5,*) SUNANG
       DOSUN=.FALSE.
       IF (SUNANG .GE. 0.0 .AND. SUNANG .LE. 80.0) DOSUN=.TRUE.
C
C      Calc total sun angle secant
       IF (DOSUN) THEN
          SUNCOS=COS(CONV*SUNANG)
          SUNSEC=1.0E+0/SUNCOS
          DO L=1,LBOT
             SZALAY=SACONV(SUNANG,ALT(L))
             SECSUN(L)=SECANG(L) + 1.0E+0/COS(SZALAY)
          ENDDO
       ENDIF
C
C      -----------------------------------
C      Calculate the fast trans predictors
C      -----------------------------------
C
       CALL CALPAR ( LBOT,
     $    RTEMP, RFAMNT, RWAMNT, ROAMNT, RCAMNT,
     $     TEMP,  FAMNT,  WAMNT,  OAMNT,  CAMNT,
     $    RPRES,  SECANG, FIXMUL, CONPRD,
     $    FPRED1, FPRED2, FPRED3, FPRED4,
     $    WPRED1, WPRED2, WPRED3, WPRED4,
     $    OPRED1, OPRED2, OPRED3, OPRED4, CPRED4 )
C
C      ----------------------------------
C      Calculate the layer transmittances
C      ----------------------------------
C
C      Calculate TAU for set 1 thru 4
C
       LTAU=.TRUE.
C
       CALL CALT1( LTAU, INDCHN, LBOT, BLMULT, NCHN1, CLIST1, LCHNON,
     $    COEF1, FIXMUL, CONPRD, FPRED1, WPRED1, OPRED1,
     $    TAU, TAUZ)
C
       CALL CALT2(       INDCHN, LBOT, BLMULT, NCHN2, CLIST2, LCHNON,
     $    COEF2, FIXMUL, CONPRD, FPRED2, WPRED2, OPRED2,
     $    TAU, TAUZ)
C
       CALL CALT3( LTAU, INDCHN, LBOT, BLMULT, NCHN3, CLIST3, LCHNON,
     $    COEF3, FIXMUL, CONPRD, FPRED3, WPRED3, OPRED3,
     $    TAU, TAUZ)
C
       CALL CALT4( LTAU, INDCHN, LBOT, BLMULT, NCHN4, CLIST4, LCHNON,
     $    COEF4, FIXMUL, CONPRD, FPRED4, WPRED4, OPRED4, CPRED4,
     $    TAU, TAUZ)
C
C
       IF (DOSUN) THEN
C         ---------------------------------------
C         Calc layer trans for effective sun path
C         ---------------------------------------
C
          IF (NREAL .GT. 0) THEN
C
             CALL CALPAR ( LBOT,
     $          RTEMP, RFAMNT, RWAMNT, ROAMNT, RCAMNT,
     $           TEMP,  FAMNT,  WAMNT,  OAMNT,  CAMNT,
     $          RPRES,  SECSUN, FIXMUL, CONPRD,
     $          FPRED1, FPRED2, FPRED3, FPRED4,
     $          WPRED1, WPRED2, WPRED3, WPRED4,
     $          OPRED1, OPRED2, OPRED3, OPRED4, CPRED4 )
C
C            Calculate TAUZSN for sets 1, 3, and 4 (but not set 2)
C
             LTAU=.FALSE.
C
             CALL CALT1( LTAU, INDCHN, LBOT, BLMULT, NCHN1, CLIST1,
     $          LSUNON, COEF1, FIXMUL, CONPRD, FPRED1, WPRED1, OPRED1,
     $          TAU, TAUZSN)
C
             CALL CALT3( LTAU, INDCHN, LBOT, BLMULT, NCHN3, CLIST3,
     $          LSUNON, COEF3, FIXMUL, CONPRD, FPRED3, WPRED3, OPRED3,
     $          TAU, TAUZSN)
C
             CALL CALT4( LTAU, INDCHN, LBOT, BLMULT, NCHN4, CLIST4,
     $          LSUNON, COEF4, FIXMUL, CONPRD, FPRED4, WPRED4, OPRED4,
     $          CPRED4, TAU, TAUZSN)
C
          ENDIF
C
          IF (NFAKE .GT. 0) THEN
C            Calc fake TAUZSN for bands 1 and 2
             CALL FAKETZ( NFAKE, INDFAK, TAUZ, SECANG(LBOT),
     $          SECSUN(LBOT), TAUZSN)
          ENDIF
C
       ELSE
C         No sun; set the sun surface-to-space trans to zero
          DO I=1,NCHAN
             TAUZSN(I)=0.0
          ENDDO
       ENDIF
C
C
C      --------------------------------------
C      Loop over radiance boundary conditions
C      --------------------------------------
 40    WRITE(6,2070)
 2070  FORMAT('Enter surface emissivity and temperature, either:',/,
     $ 'A) emissivity   temperature, or',/,
     $ 'B) emissivity  -1 (offset flag)  bottom layer temp offset')
       READ(5,9000) CLINE
       READ(CLINE,*) EBOT(1), TSURF
       IF (TSURF .LT. 0.0) THEN
          READ(CLINE,*) EBOT(1), TSURF, OFFSET
          TSURF=TEMP(LBOT) + OFFSET
          IF (TSURF .LT. 0.0) TSURF=0.0
       ENDIF
       IF (EBOT(1) .LT. 0.0) EBOT(1)=0.0
       IF (EBOT(1) .GT. 1.0) EBOT(1)=1.0
C
C      Assign Lambertian thermal reflectance
C      Note: the *thermal* reflectance should probably always be
C      Lambertian.  The reflected thermal is based upon an integral
C      of downwelling atmos thermal rad coming in to the surface at
C      all angles 0 to 90 degrees.  This integral was evaluated
C      for Lambertian reflectance.  Using anything else is at best
C      inconsistent.
       RHOTHR(1)=(1.0 - EBOT(1))/PI
C
C      Assign solar reflectance
C      Note: the solar reflectance does not need to be Lambertian.
C      For the purposes of this demo program, we'll be boring
C      and stick to Lambertian.
       RHOSUN(1)=RHOTHR(1)
C
       DO I=2,NCHAN
          EBOT(I)=EBOT(1)
          RHOTHR(I)=RHOTHR(1)
          RHOSUN(I)=RHOSUN(1)
       ENDDO
C
C      ----------------------
C      Calculate the radiance
C      ----------------------
C
       CALL CALRAD ( NCHAN, FREQ, TAU, TEMP, TSURF, EBOT, LBOT,
     $    SUNCOS, RHOSUN, DISTES, HSUN, TAUZSN,
     $    SECANG(LBOT), RHOTHR, LABOVE, COEFF, TAUZ, RAD, BT)
C
C      -------------------
C      Output the radiance
C      -------------------
       WRITE(6,3000)
 3000  FORMAT('Enter name of radiance output file to create')
       READ(5,9000) CLINE
C      Open the output file for radiance
       OPEN(UNIT=10,FILE=CLINE,FORM='FORMATTED',STATUS='NEW')
C
C      Write the header for the output text file
       WRITE(10,3020) PNAM
 3020  FORMAT('! profile = ',A40)
       WRITE(10,3030) SVA,SECANG(LBOT),SUNANG
 3030  FORMAT('! NAST viewing angle=',F6.2,' (path mult=',
     $    F6.3,'), sun angle=',F6.2)
       WRITE(10,3040) LBOT, EBOT(1), TSURF, OFFSET
 3040  FORMAT('! Bottom layer=',I3,', e surface=',F5.3,
     $    ', T surface=',F6.2,',  T contrast=',F6.2)
       WRITE(10,3050)
 3050  FORMAT('! chan freq  BT',/,
     $        '! --------  ------')
C
C      Write out the radiance and then close the output file
       DO I=1,NCHAN
          WRITE(10,3060) FREQ(I), BT(I)
 3060     FORMAT(F10.4,' ',0PF7.2)
 210   ENDDO
       CLOSE(10)
C
C      ------------------------------------------
C      End loop over radiance boundary conditions
C      ------------------------------------------
       WRITE(6,3110)
 3110  FORMAT('Do you want to redo the rad calc with',
     $    ' different boundary conditions? (y/n)')
       READ(5,9100) YNANS
 9100  FORMAT(A1)
       IF (YNANS .EQ. 'y' .OR. YNANS .EQ. 'Y') GOTO 40
C
C      ----------------------------
C      End loop over viewing angles
C      ----------------------------
       WRITE(6,3210)
 3210  FORMAT('Do you want to redo the profile with a',
     $    ' different viewing angle? (y/n)')
       READ(5,9100) YNANS
       IF (YNANS .EQ. 'y' .OR. YNANS .EQ. 'Y') GOTO 30
C
C      ----------------------
C      End loop over profiles
C      ----------------------
       WRITE(6,3310)
 3310  FORMAT('Do you want to do another profile? (y/n)')
       READ(5,9100) YNANS
       IF (YNANS .EQ. 'y' .OR. YNANS .EQ. 'Y') GOTO 20
C
       STOP
       END
